#include "prefix.glsl"
#include "srgb.glsl"
#if TONE_MAPPING
#include "tone_map.glsl"
#endif
#line 6


VARYING(vec4, v_color)


#if NUM_TU > 0	
	VARYING(vec2, v_texcoord0)
#endif
#if NUM_TU > 1
	VARYING(vec2, v_texcoord1)
#endif
#if FOG
	VARYING(float, v_fog)
#endif

#if VSHADER

	ATTRIBUTE(vec4, a_vertex)
	ATTRIBUTE(vec4, a_color)
	
	#if LIT
	ATTRIBUTE(vec3, a_normal)
	#endif
	
	#if NUM_TU > 0
		ATTRIBUTE(vec2, a_texcoord0)
	#endif
	#if NUM_TU > 1
		ATTRIBUTE(vec2, a_texcoord1)
	#endif
	

	void main (void)
	{
		STD_BROADCAST
		gl_Position = u_mvp_matrix * (a_vertex - vec4(u_mv_offset,0.0));
		#if !HAS_CORE
		gl_ClipVertex = u_modelview_matrix * (a_vertex - vec4(u_mv_offset,0.0));
		#endif
		
		#if LIT && GAMMA && !HAS_CORE
			vec3 normal_eye = mat3(u_modelview_matrix) * a_normal;

			v_color.rgb = /*to_srgb*/(a_color.rgb) * gl_LightSource[0].diffuse.rgb * max(0.0, dot(normal_eye, gl_LightSource[0].position.xyz));
			v_color.a = a_color.a;

		#elif LIT && !HAS_CORE
			vec3 normal_eye = mat3(u_modelview_matrix) * a_normal;

			v_color.rgb = a_color.rgb * gl_LightSource[0].diffuse.rgb * max(0.0, dot(normal_eye, gl_LightSource[0].position.xyz));
			v_color.a = a_color.a;

		#else		
			v_color = a_color;
		#endif
		
		#if NUM_TU > 0
			v_texcoord0 = a_texcoord0.st;
		#endif
		#if NUM_TU > 1
			v_texcoord1 = a_texcoord1.st;
		#endif
		
		#if FOG && !HAS_CORE
			float eye_z = (u_modelview_matrix * (a_vertex - vec4(u_mv_offset,0.0))).z;
			v_fog = (gl_Fog.end + eye_z) * gl_Fog.scale;
		#endif		
		
		#if TONE_MAPPING
			calc_light_meter();
		#endif
	}

#endif

#if FSHADER

	uniform sampler2D	u_tex0;
	uniform sampler2D	u_tex1;
//	uniform	sampler2D	u_dither;
	
	void main()
	{
		#if GAMMA && LIT
		
			#if NUM_TU == 0	
				gl_FragColor = v_color;
			#elif NUM_TU == 1
				vec4 tex = texture2D(u_tex0, v_texcoord0);				
				gl_FragColor = v_color * vec4((tex.rgb),tex.a);
			#elif NUM_TU == 2
				vec4 tex = texture2D(u_tex0, v_texcoord0) * texture2D(u_tex1,v_texcoord1);
				gl_FragColor = v_color * vec4((tex.rgb),tex.a);
			#endif

		#else

			#if NUM_TU == 0	
				gl_FragColor = v_color;
			#elif NUM_TU == 1
				gl_FragColor = v_color * texture2D(u_tex0, v_texcoord0);
			#elif NUM_TU == 2
				gl_FragColor = v_color * texture2D(u_tex0, v_texcoord0) * texture2D(u_tex1, v_texcoord1);
			#endif
		#endif
		
		// "test mode" - equiv is > 0
		#if ALPHA_TEST == 1
			if(gl_FragColor.a == 0.0)
				discard;
		#endif
		
		// "keep solid mode" - equiv is >= 1
		#if ALPHA_TEST == 2
			if(gl_FragColor.a != 1.0)
				discard;
		#endif

//		float dither = texture2D(u_dither, gl_FragCoord.xy * 0.125).a;		
//		gl_FragColor.rgb += (dither - 0.5) * 0.0078431372549;
		
		#if GAMMA
			gl_FragColor.rgb = to_linear(gl_FragColor.rgb);
		#endif
		
		#if FOG && !HAS_CORE
			gl_FragColor.rgb = mix(gl_Fog.color.rgb,gl_FragColor.rgb, clamp(v_fog,0.0,1.0));
		#endif

		#if TONE_MAPPING
			tone_map_srgb(gl_FragColor.rgb);
		#endif
	}
	

#endif
