#line 1

#ifndef WANT_LIGHTMETER
#define WANT_LIGHTMETER 0
#endif

#include "srgb.glsl"
#line 8

#if !defined(VSHADER_TONE_ONLY)

VARYING(float, v_tonemap_gain)

#endif

uniform sampler2D	u_tex_tone;

#define bias 0.75
	
#if WANT_LIGHTMETER
VARYING(vec2, amp_meter)
#endif

#if USE_UBOS
	#if TONE_MODE == MODE_TONE_DYNAMIC
		#define tonemap_gain u_tonemap_gain
	#else
		#define tonemap_gain vec4(0, 0, 0, 3.2)
	#endif	
#else
	uniform vec4 u_tonemap_gain;
	#define tonemap_gain u_tonemap_gain
#endif

float A = 0.15;
float B = 0.50;
float C = 0.10;
float D = 0.20;

float E = 0.02;
float F = 0.30;
vec3 W = vec3(3.2);

vec3 Uncharted2Tonemap(vec3 x)
{
	return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
}

float	R_white = 3.0;
vec3 Reinhard(vec3 L)
{
	vec3 num = L * (L/(R_white*R_white) + 1.0);
	vec3 den = L + 1.0;
	return num/den;
}

		// If we are going to tone map using Alex's epic equation,
		// go to sRGB-ish space first; his stuff is tuned to sRGB,
		// not linear.



void tone_map_uncharted(inout vec3 rgb, in float gain)
{

//			dumb clamping
//			rgb = clamp(rgb,0.0,1.0);

//			naive coolor Reinhardt - needs srgb
//			rgb = Reinhard(rgb);

//			Dawson - no srgb
//			vec3 x = max(rgb-0.004,0.0);
//			rgb = (x*(6.2*x+.5))/(x*(6.2*x+1.7)+0.06);
		
// uncharted 2
		float ExposureBias = gain;
		vec3 curr = Uncharted2Tonemap(ExposureBias*rgb);
		vec3 whiteScale = 1.0f/Uncharted2Tonemap(vec3(tonemap_gain.w));
		rgb = curr*whiteScale;
}

void tone_map_gifford(inout vec3 rgb, in float gain)
{
//	rgb = to_srgb(rgb);
	vec3 blend_rat = pow(clamp(rgb,0.0,1.0),vec3(bias));
	rgb = mix(rgb,1.0 - pow(clamp(1.0 - rgb,0.0,1.0), vec3(gain)),blend_rat);
}




#if VSHADER

#if !defined(VSHADER_TONE_ONLY)
void	calc_light_meter()
{
	// Tone mapping light meter.
	// This samples the dynamic exposure texture (a 1x1 result of much copying)
	// and figures out how much gain to apply.  Hoisted up into the vertex shader
	// to cut ops out of fragment processing - the tone map levels do not vary
	// by frame position!!

	// The light meter texture contains the _immediate_ brightness in "r" and
	// a blended value in "g" - the result is that g assymptotically r.
	// By sampling the max of both, we take the brighter of what we see now or
	// what we did see, for a fast adjustment when things get bright but slow
	// adjustment when things get dim.
	
	vec2 light_meter_raw = texture2DLod(u_tex_tone,vec2(0.5,0.5),0).rg;
	float	overall_color = max(light_meter_raw.r,light_meter_raw.g);

	float amp = (overall_color);
	float amp_lim = clamp(amp,tonemap_gain.y,tonemap_gain.z);
	float total_gain = tonemap_gain.x / amp_lim;
	v_tonemap_gain = total_gain;
	
	#if WANT_LIGHTMETER			
		amp_meter = vec2(amp,amp_lim);			
	#endif
}
#endif

void vshader_tone_map(inout vec3 rgb)
{
	vec2 light_meter_raw = texture2DLod(u_tex_tone,vec2(0.5,0.5),0).rg;
	float	overall_color = max(light_meter_raw.r,light_meter_raw.g);

	float amp = (overall_color);
	float amp_lim = clamp(amp,tonemap_gain.y,tonemap_gain.z);
	float total_gain = tonemap_gain.x / amp_lim;

	tone_map_uncharted(rgb, total_gain);
}

void vshader_tone_map_srgb(inout vec3 rgb)
{
	rgb = to_linear(rgb);
	vshader_tone_map(rgb);
	rgb = to_srgb(rgb);
}

#endif

#if FSHADER && !defined(VSHADER_TONE_ONLY)


void	tone_map(inout vec3 rgb)
{
	tone_map_uncharted(rgb, v_tonemap_gain);

	// And now...Alex's epic equation. v_tonemap_gain is a gain
	// ratio (e.g. 1.0 is unity) and bias is a fixed constant
	// for where in the brightness curve we start to squash.
	// Generally lower biases squash lower brightness levels and
	// as a result make tings more, um, "crispy".

	
	// Light meter code - just for debugging - not particularly
	// well optimized.
	
	#if WANT_LIGHTMETER

		if(v_texcoord0.t < 0.05)
		{
			float total_gain_log = log2(v_tonemap_gain);
			float amp_percent = (total_gain_log / 16.0) + 0.5;
			float pixel_pos = v_texcoord0.s;
			if(amp_percent < 0.5)
			{
				if(amp_percent > pixel_pos)
					rgb = vec3(1,0,0);
			}
			else
			{
				if(amp_percent > pixel_pos)
					rgb = vec3(0,1,0);
			}
		}
		else if (v_texcoord0.t < 0.1)
		{
			float amp_log = log2(amp_meter.x);
			float amp_lim_log = log2(amp_meter.y);
			float meter_percent = (amp_log / 16.0) + 0.5;
			float pixel_pos = v_texcoord0.s;
			if(amp_lim_log != amp_log)
			{
				if(meter_percent > pixel_pos)
					rgb = vec3(1,0,0);
			}
			else
			{
				if(meter_percent > pixel_pos)
					rgb = vec3(0,1,0);
			}
		}
	#endif	
	
}

void	tone_map_srgb(inout vec3 rgb)
{
	rgb = to_linear(rgb);
	tone_map(rgb);
	rgb = to_srgb(rgb);
}

void	tone_map_srgb_to_linear(inout vec3 rgb)
{
	rgb = to_linear(rgb);
	tone_map(rgb);
}

void	tone_map_linear_to_srgb(inout vec3 rgb)
{
	tone_map(rgb);
	rgb = to_srgb(rgb);
}


#endif
